-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

AbilityManager = {}
AbilityManager.name = "AbilityManager"
AbilityManager.version = "$Id$"
AbilityManager.abilityByID = {}
AbilityManager.abilityByName = {}
AbilityManager.abilityCallbacks = {}
AbilityManager.abilityCallbackMap = {}
AbilityManager.callbackID = 0
AbilityManager.abilityCount = 0
AbilityManager.beginType = 1
AbilityManager.endType = 2
AbilityManager.availableType = 3
AbilityManager.notAvailableType = 4

--
-- Main functions
--
local function AbilityDetail(abilityID)
	if Inspect.Ability.New ~= nil then
		return Inspect.Ability.New.Detail(abilityID)
	else
		return Inspect.Ability.Detail(abilityID)
	end
end

local function AbilityList()
	if Inspect.Ability.New ~= nil then
		return Inspect.Ability.New.List()
	else
		return Inspect.Ability.List()
	end
end

function AbilityManager.Initialise(addonName)
	if Event.Ability.New ~= nil then
		table.insert(Event.Ability.New.Add, { AbilityManager.ClearIDs, addonName, "AbilityAdd" })
		table.insert(Event.Ability.New.Remove, { AbilityManager.ClearIDs, addonName, "AbilityRemove" })
		table.insert(Event.Ability.New.Cooldown.Begin, { AbilityManager.AbilityBegin, addonName, "AbilityBegin" })
		table.insert(Event.Ability.New.Cooldown.End, { AbilityManager.AbilityEnd, addonName, "AbilityEnd" })
	else
		table.insert(Event.Ability.Add, { AbilityManager.ClearIDs, addonName, "AbilityAdd" })
		table.insert(Event.Ability.Remove, { AbilityManager.ClearIDs, addonName, "AbilityRemove" })
		table.insert(Event.Ability.Cooldown.Begin, { AbilityManager.AbilityBegin, addonName, "AbilityBegin" })
		table.insert(Event.Ability.Cooldown.End, { AbilityManager.AbilityEnd, addonName, "AbilityEnd" })
	end
end

function AbilityManager.Clear()
	AbilityManager.abilityCallbacks = {}
	AbilityManager.abilityCallbackMap = {}
	AbilityManager.ClearIDs()
	AbilityManager.abilityCount = 0
end

function AbilityManager.ClearIDs()
	AbilityManager.abilityByID = {}
	AbilityManager.abilityByName = {}
end

function AbilityManager.MapAbility(abilityID, detail)
	if detail and detail.name and detail.cooldown then
		AbilityManager.abilityByID[abilityID] = detail.name
		local abilityDetails = {}
		abilityDetails.id = abilityID
		abilityDetails.name = detail.name
		abilityDetails.duration = Utilities.Round(detail.cooldown, 2)
		AbilityManager.abilityByName[detail.name] = abilityDetails
	end
end

function AbilityManager.GetName(abilityID)
	if not AbilityManager.abilityByID[abilityID] then
		local detail = AbilityDetail(abilityID)
		AbilityManager.MapAbility(abilityID, detail)
	end
	
	return AbilityManager.abilityByID[abilityID]
end

function AbilityManager.GetDuration(abilityName)
	if AbilityManager.abilityByName[abilityName] then
		return AbilityManager.abilityByName[abilityName].duration
	end
	
	return -1
end

function AbilityManager.GetID(abilityName)
	if not AbilityManager.abilityByName[abilityName] then
		local abilities = AbilityList()
		if abilities then
			local details = AbilityDetail(abilities)
			for abilityID, detail in pairs(details) do
				AbilityManager.MapAbility(abilityID, detail)
			end
		end
	end
	
	if AbilityManager.abilityByName[abilityName] then
		return AbilityManager.abilityByName[abilityName].id
	else
		return nil
	end
end

function AbilityManager.AddAbilityBeginCallback(abilityName, callback)
	return AbilityManager.InternalAddAbilityCallback(abilityName, callback, AbilityManager.beginType)
end

function AbilityManager.AddAbilityEndCallback(abilityName, callback)
	return AbilityManager.InternalAddAbilityCallback(abilityName, callback, AbilityManager.endType)
end

function AbilityManager.AddAbilityAvailableCallback(abilityName, callback)
	return AbilityManager.InternalAddAbilityCallback(abilityName, callback, AbilityManager.availableType)
end

function AbilityManager.AddAbilityNotAvailableCallback(abilityName, callback)
	return AbilityManager.InternalAddAbilityCallback(abilityName, callback, AbilityManager.notAvailableType)
end

function AbilityManager.RemoveAbilityCallback(id)
	local key = AbilityManager.abilityCallbackMap[id]
	AbilityManager.abilityCallbacks[key][id] = nil
	AbilityManager.abilityCount = AbilityManager.abilityCount - 1
end

function AbilityManager.InternalAddAbilityCallback(abilityName, callback, eventType)
	if not abilityName then
		print("Ignoring ability with no name")
		return -1
	end
	if not abilityName then
		print("Ignoring " .. unitSpec .. " ability with no name")
		return -1
	end
	
	local abilityID = AbilityManager.GetID(abilityName)
	
	AbilityManager.abilityCount = AbilityManager.abilityCount + 1
	
	local id = AbilityManager.GetCallbackID()
	AbilityManager.abilityCallbackMap[id] = abilityName
	
	if not AbilityManager.abilityCallbacks[abilityName] then
		AbilityManager.abilityCallbacks[abilityName] = {}
	end

	AbilityManager.abilityCallbacks[abilityName][id] = {}
	AbilityManager.abilityCallbacks[abilityName][id].callback = callback
	AbilityManager.abilityCallbacks[abilityName][id].eventType = eventType

	return id
end

function AbilityManager.GetCallbackID()
	AbilityManager.callbackID = AbilityManager.callbackID + 1
	return "#" .. AbilityManager.callbackID
end

function AbilityManager.Refresh()
	if AbilityManager.abilityCount > 0 then
		local currentTime = Inspect.Time.Frame()
		local abilities = AbilityList()
		if abilities then
			local details = AbilityDetail(abilities)
			if details then
				for abilityID, detail in pairs(details) do
					local abilityCallbacks = AbilityManager.abilityCallbacks[detail.name]
					if abilityCallbacks and AbilityManager.abilityByName[detail.name] then
						local isStarted = false
						local isEnded = false
						if not AbilityManager.abilityByName[detail.name].endTime then
							isEnded = true
						elseif AbilityManager.abilityByName[detail.name].endTime <= currentTime + 0.05 then
							isEnded = true
							AbilityManager.abilityByName[detail.name].endTime = nil
						else
							isStarted = true
						end
						
						for id, callbackTable in pairs(abilityCallbacks) do
							if callbackTable.eventType == AbilityManager.availableType and isEnded == true then
								callbackTable.callback(abilityName)
							elseif callbackTable.eventType == AbilityManager.notAvailableType and isStarted == true then
								callbackTable.callback(abilityName)
							end
						end
					end
				end
			end
		end
	end
end

function AbilityManager.AbilityFired(abilities)
	if AbilityManager.abilityCount > 0 then
		local currentTime = 0
		
		for abilityID, count in pairs(abilities) do
			local abilityName = AbilityManager.GetName(abilityID)
			if abilityName and AbilityManager.abilityCallbacks[abilityName] then
				if currentTime == 0 then
					currentTime = Inspect.Time.Frame()
				end
				
				local duration = AbilityManager.GetDuration(abilityName)
				local thisDuration = Utilities.Round(count, 2)
				local isStarted = false
				if duration == thisDuration then
					AbilityManager.abilityByName[abilityName].endTime = currentTime + count
					isStarted = true
				end
				
				local isEnded = false
				if thisDuration == 0 and AbilityManager.abilityByName[abilityName].endTime and AbilityManager.abilityByName[abilityName].endTime <= currentTime + 0.05 then
					isEnded = true
					AbilityManager.abilityByName[abilityName].endTime = nil
				elseif AbilityManager.abilityByName[abilityName].endTime and AbilityManager.abilityByName[abilityName].endTime <= currentTime + 0.05 then
					AbilityManager.abilityByName[abilityName].endTime = currentTime + count
				end
				
				for id, callbackTable in pairs(AbilityManager.abilityCallbacks[abilityName]) do
					if callbackTable.eventType == AbilityManager.beginType and isStarted == true then
						callbackTable.callback(abilityName)
					elseif callbackTable.eventType == AbilityManager.endType and isEnded == true then
						callbackTable.callback(abilityName)
					elseif callbackTable.eventType == AbilityManager.availableType and isEnded == true then
						callbackTable.callback(abilityName)
					elseif callbackTable.eventType == AbilityManager.notAvailableType and isStarted == true then
						callbackTable.callback(abilityName)
					end
				end
			end
		end
	end
end

--
-- Event Handlers
--

function AbilityManager.AbilityBegin(abilities)
	AbilityManager.AbilityFired(abilities)
end

function AbilityManager.AbilityEnd(abilities)
	AbilityManager.AbilityFired(abilities)
end

