-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--

local function Redraw(widget)
	widget:OldSetWidth(widget.width)
	widget:OldSetHeight(widget.height)
	widget.textureFrame:SetWidth(widget.width - 2 * widget.margin)
	widget.textureFrame:SetHeight(widget.height - 2 * widget.margin)
	widget.mouseoverFrame:SetWidth(widget.width - 1 * widget.margin)
	widget.mouseoverFrame:SetHeight(widget.height - 1 * widget.margin)
end

local function Resize(widget)
	widget.margin = math.ceil(widget.textureFrame:GetWidth() / 16)
	widget.width = widget.textureFrame:GetTextureWidth() + 2 * widget.margin
	widget.height = widget.textureFrame:GetTextureHeight() + 2 * widget.margin
	Redraw(widget)
end

local function SetTexture(widget, textureAddon, textureName)
	widget.textureFrame:SetTexture(textureAddon, textureName)
	Resize(widget)
end

local function GetTexture(widget)
	return widget.textureFrame:GetTexture()
end

local function MouseDown(widget)
	if widget.enabled == false then
		return
	end
	
	local shrinkX = math.floor(widget.width / 8)
	if shrinkX < 1 then shrinkX = 1 end
	local shrinkY = math.floor(widget.height / 8)
	if shrinkY < 1 then shrinkY = 1 end
	widget.textureFrame:SetWidth(widget.width - 2 * widget.margin - shrinkX)
	widget.textureFrame:SetHeight(widget.height - 2 * widget.margin - shrinkY)
end

local function MouseUp(widget, fireEvent)
	if widget.enabled == false then
		return
	end
	
	widget.textureFrame:SetWidth(widget.width - 2 * widget.margin)
	widget.textureFrame:SetHeight(widget.height - 2 * widget.margin)
	
	if fireEvent == true and widget.Event.LeftPress then
		widget.Event.LeftPress(widget)
	end
end

local function SetHighlightVisible(widget, visible, fireEvent)
	if widget.enabled == false then
		widget.mouseoverFrame:SetVisible(false)
		return
	end
	
	local oldVisible = widget.mouseoverFrame:GetVisible()
	widget.mouseoverFrame:SetVisible(visible)
	
	if fireEvent == true and oldVisible ~= visible then
		if visible == true then
			if widget.Event.MouseIn then
				widget.Event.MouseIn(widget)
			end
		else
			if widget.Event.MouseOut then
				widget.Event.MouseOut(widget)
			end
		end
	end
end

local function SetVisible(widget, visible)
	if visible == false then
		SetHighlightVisible(widget, false, false)
	else
		local mouse = Inspect.Mouse()
		local mouseDown = false
		if mouse.x and mouse.y then
			local x = widget:GetLeft()
			local y = widget:GetTop()
			if mouse.x >= x and mouse.x <= x + widget.width and
			   mouse.y >= y and mouse.y <= y + widget.height then
				SetHighlightVisible(widget, true, true)
				
				if mouse.left == true then
					mouseDown = true
				end
			else
				SetHighlightVisible(widget, false, true)
			end
		end
		
		if mouseDown == true then
			MouseDown(widget)
		else
			MouseUp(widget, false)
		end
	end
	
	widget:OldSetVisible(visible)
end

local function SetEnabled(widget, enabled)
	widget.enabled = enabled
end

local function GetEnabled(widget)
	return widget.enabled
end

local function GetTextureHeight(widget)
	return widget.textureFrame:GetTextureHeight()
end

local function GetTextureWidth(widget)
	return widget.textureFrame:GetTextureWidth()
end

local function ResizeToTexture(widget)
	widget.textureFrame:ResizeToTexture()
	Resize(widget)
end

local function SetWidth(widget, width)
	widget.width = width
	widget.margin = math.ceil(width / 16)
	Redraw(widget)
end

local function SetHeight(widget, height)
	widget.height = height
	widget.margin = math.ceil(height / 16)
	Redraw(widget)
end

function Library.LibSimpleWidgetsExt.TextureButton(name, parent)
	
	local widget = UI.CreateFrame("Texture", name, parent)
	
	widget.enabled = true
	widget.margin = 2
	
	widget:SetTexture("LibSimpleWidgetsExt", "Textures/ButtonBorder.png")
	widget:SetMouseMasking("limited")
	widget.Event.MouseIn = function () SetHighlightVisible(widget, true, true) end
	widget.Event.MouseOut = function () MouseUp(widget, false) SetHighlightVisible(widget, false, true) end
	widget.Event.LeftUp = function () MouseUp(widget, true) end
	widget.Event.LeftDown = function () MouseDown(widget) end

	widget.textureFrame = UI.CreateFrame("Texture", widget:GetName().."Button", widget)
	widget.textureFrame:SetPoint("CENTER", widget, "CENTER")

	widget.mouseoverFrame = UI.CreateFrame("Texture", widget:GetName().."Mouseover", widget)
	widget.mouseoverFrame:SetLayer(widget.textureFrame:GetLayer() + 1)
	widget.mouseoverFrame:SetPoint("CENTER", widget, "CENTER")
	widget.mouseoverFrame:SetTexture("LibSimpleWidgetsExt", "Textures/ButtonMouseOver.png")
	
	widget.width = widget:GetWidth()
	widget.height = widget:GetHeight()
	widget.textureFrame:SetWidth(widget.width - 2 * widget.margin)
	widget.textureFrame:SetHeight(widget.height - 2 * widget.margin)
	widget.mouseoverFrame:SetWidth(widget.width - 1 * widget.margin)
	widget.mouseoverFrame:SetHeight(widget.height - 1 * widget.margin)
	
	widget.OldSetVisible = widget.SetVisible
	widget.OldSetHeight = widget.SetHeight
	widget.OldSetWidth = widget.SetWidth
	widget.OldGetTextureHeight = widget.GetTextureHeight
	widget.OldGetTextureWidth = widget.GetTextureWidth
	widget.OldResizeToTexture = widget.ResizeToTexture
	
	widget.SetVisible = SetVisible
	widget.SetTexture = SetTexture
	widget.GetTexture = GetTexture
	widget.SetWidth = SetWidth
	widget.SetHeight = SetHeight
	widget.GetTextureHeight = GetTextureHeight
	widget.GetTextureWidth = GetTextureWidth
	widget.ResizeToTexture = ResizeToTexture
	widget.SetEnabled = SetEnabled
	widget.GetEnabled = GetEnabled
	
	Library.LibSimpleWidgets.EventProxy(widget, {"LeftPress"})
	Library.LibSimpleWidgets.EventProxy(widget, {"MouseIn"})
	Library.LibSimpleWidgets.EventProxy(widget, {"MouseOut"})

	SetVisible(widget, widget:GetVisible())
	
	return widget
end

