-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- This file provide a numeric label coupled with a RiftSlider to allow a user to select a number
-- within a given range
--

--
-- Global variables
--

NumberSlider = {}
NumberSlider.debug = true
NumberSlider.version = "1.0"
NumberSlider.name = "NumberSlider"

--
-- Main functions
--

function NumberSlider.Create(parentFrame, rangeMin, rangeMax, stepSize, initialValue, changedCallback)
	if (rangeMin < 0) then
		print(string.format("%s: slider range minimum (%d) must be positive", NumberSlider.name, rangeMin))
		return nil
	end
	if (rangeMax <= rangeMin) then
		print(string.format("%s: slider range maximum (%d) must be greater than range minimum (%d)", NumberSlider.name, rangeMax, rangeMin))
		return nil
	end
	if ((initialValue < rangeMin) or (initialValue > rangeMax)) then
		print(string.format("%s: slider initial value (%d) must be within the range %d-%d", NumberSlider.name, initialValue, rangeMin, rangeMax))
		return nil
	end
	if (stepSize < 0) or (stepSize > rangeMax - rangeMin) then
		print(string.format("%s: slider step size (%d) must be positive and less than the range", NumberSlider.name, stepSize))
		return nil
	end
	
	local numberSlider = {}
	numberSlider.parentFrame = parentFrame
	numberSlider.rangeMin = rangeMin
	numberSlider.rangeMax = rangeMax
	numberSlider.stepSize = stepSize
	numberSlider.changedCallback = changedCallback
	
	numberSlider.numberFrame = UI.CreateFrame("Text", "NumberSliderText", parentFrame)
	
	numberSlider.sliderFrame = UI.CreateFrame("RiftSlider", "NumberSliderSlider", parentFrame)
	numberSlider.sliderFrame:SetRange(0, NumberSlider.GetUnscaledValue(numberSlider, rangeMax))
	numberSlider.sliderFrame:SetPoint("BOTTOMLEFT", parentFrame, "BOTTOMLEFT", 10, 0)
	numberSlider.sliderFrame:SetPoint("BOTTOMRIGHT", parentFrame, "BOTTOMRIGHT", -10, 0)

	NumberSlider.SetValue(numberSlider, initialValue)

	numberSlider.sliderFrame.Event.SliderChange = function () NumberSlider.SliderChanged(numberSlider) end
	
	return numberSlider
end

function NumberSlider.SetValue(numberSlider, newValue, fireChangedEvent)
	if not numberSlider then
		print(string.format("%s: cannot SetValue on a nil slider", NumberSlider.name))
		return
	end
	if ((newValue < numberSlider.rangeMin) or (newValue > numberSlider.rangeMax)) then
		print(string.format("%s: slider initial value (%d) must be within the range %d-%d", NumberSlider.name, newValue, numberSlider.rangeMin, numberSlider.rangeMax))
		return nil
	end
	
	NumberSlider.InternalSetText(numberSlider, newValue)
	numberSlider.sliderFrame:SetPosition(NumberSlider.GetUnscaledValue(numberSlider, newValue))

	if fireChangedEvent == true then
		if numberSlider.changedCallback then
			numberSlider.changedCallback(newValue)
		end
	end
end

function NumberSlider.SliderChanged(numberSlider)
	if not numberSlider then
		print(string.format("%s: cannot SliderChanged on a nil slider", NumberSlider.name))
		return
	end
	
	local newValue = NumberSlider.GetScaledValue(numberSlider,numberSlider.sliderFrame:GetPosition())
	NumberSlider.SetValue(numberSlider, newValue, true)
end

function NumberSlider.InternalSetText(numberSlider, newValue)
	numberSlider.value = newValue
	numberSlider.numberFrame:SetText(tostring(newValue))
	local offset = numberSlider.parentFrame:GetWidth() / 2 - numberSlider.numberFrame:GetWidth() / 2
	if offset < 0 then
		offset = 0
	end
	numberSlider.numberFrame:SetPoint("TOPLEFT", numberSlider.parentFrame, "TOPLEFT", offset, 0)
end

function NumberSlider.GetValue(numberSlider)
	if not numberSlider then
		print(string.format("%s: cannot GetValue on a nil slider", NumberSlider.name))
		return
	end
	
	return numberSlider.value
end

function NumberSlider.GetScaledValue(numberSlider, value)
	return numberSlider.rangeMin + value * numberSlider.stepSize
end

function NumberSlider.GetUnscaledValue(numberSlider, value)
	return math.floor((value - numberSlider.rangeMin) / numberSlider.stepSize)
end

function NumberSlider.SetEnabled(numberSlider, enabled)
	numberSlider.sliderFrame:SetEnabled(enabled)
end
